import sys
import argparse

from traffic_light.error import TrafficLightError, MultipleTrafficLightsError
from traffic_light.core import ClewareTrafficLight, State, Color


def main(args=sys.argv[1:]):
    parser = argparse.ArgumentParser(
        prog="traffic_light",
        description='Turns the led of the cleware traffic light on or off'
    )
    parser.add_argument('-r', '--red',
                        choices=["on", "off"],
                        help="Controlls the red led")
    parser.add_argument('-y', '--yellow',
                        choices=["on", "off"],
                        help="Controlls the yellow led")
    parser.add_argument('-g', '--green',
                        choices=["on", "off"],
                        help="Controlls the green led")
    parser.add_argument('-a', '--all',
                        choices=["on", "off"],
                        help="Controlls all leds")
    parser.add_argument('-adr', '--address',
                        type=int,
                        help="Specifies which traffic light should be used")
    args = parser.parse_args(args)

    if not vars(args)['all']:
        all_given_colors = [(k, v) for k, v in vars(args).items()
                            if v is not None and k is not "address"]
        if not all_given_colors:
            parser.print_help()
            return 1
    else:
        all_state = vars(args)["all"]
        all_given_colors = [
            ('red', all_state), ('yellow', all_state), ('green', all_state)
        ]

    for color_name, state_name in all_given_colors:
        color = Color[color_name.upper()]
        state = State[state_name.upper()]
        address = vars(args)["address"]
        try:
            ClewareTrafficLight(address).set_led(color, state)
        except MultipleTrafficLightsError as exc:
            print(exc, file=sys.stderr)
            ClewareTrafficLight.print_devices()
            return 1
        except TrafficLightError as exc:
            print("Error -> {}".format(exc.message), file=sys.stderr)
            return 1

    return 0


if __name__ == '__main__':
    sys.exit(main())



from enum import IntEnum

import usb.core
import usb.util

from traffic_light.error import TrafficLightError, MultipleTrafficLightsError

CTRL_ENDPOINT = 0x02
ID_VENDOR = 0x0d50
ID_PRODUCT = 0x0008
INTERFACE = 0


class Color(IntEnum):
    RED = 0x10
    YELLOW = 0x11
    GREEN = 0x12


class State(IntEnum):
    OFF = 0x0
    ON = 0x1


class ClewareTrafficLight:
    def __init__(self, address=None):
        if address:
            self.address = address
            self.device = usb.core.find(
                address=address,
                idVendor=ID_VENDOR,
                idProduct=ID_PRODUCT)
        elif len(list(ClewareTrafficLight.find_devices())) > 1:
            raise MultipleTrafficLightsError(
                "No address is given and there are multiple devices conected! "
                "Use 'print_devices' to see a list of connected devices."
            )
        else:
            self.device = usb.core.find(
                idVendor=ID_VENDOR,
                idProduct=ID_PRODUCT)
        if self.device is None:
            raise TrafficLightError('Cleware traffic light not found!')

    @staticmethod
    def find_devices():
        """Returns the raw iterator of all found traffic lights"""
        devices = usb.core.find(find_all=True, idVendor=ID_VENDOR, idProduct=ID_PRODUCT)
        if devices:
            return devices
        return []

    @staticmethod
    def print_devices():
        """Prints a list of all connected traffic lights"""
        devices = ClewareTrafficLight.get_devices()
        for device in devices:
            print(device)

    @staticmethod
    def get_devices():
        """Returns a list of ClewareTrafficLight instances"""
        usb_devices = ClewareTrafficLight.find_devices()
        return [ClewareTrafficLight(d.address) for d in usb_devices]

    def set_led(self, color, value):
        """Sets the given state and color of the attached traffic light

        Attribute:
            color -- the to set color as the enum. E.g. Color.RED
            state -- the state to which it should be set. E.g. State.ON
            address -- the usb address of a specific traffic light
        """
        try:
            self.device.write(CTRL_ENDPOINT, [0x00, color, value])
        except Exception as exc:
            raise TrafficLightError(str(exc)) from exc
        finally:
            usb.util.dispose_resources(self.device)

    def __getattr__(self, name):
        """Parses attribut calls in function"""
        args = name.split('_')
        try:
            color = Color[args[0].upper()]
            state = State[args[1].upper()]
        except:
            raise TrafficLightError("Either the given color or state could not be parsed")
        return lambda: self.set_led(color, state)

    def __str__(self):
        """Converts instance into string with important imformations"""
        return ("== Cleware Traffic Light ==\n"
                "Address: {} \n"
                "IdVendor: {} \n"
                "IdProduct: {}".format(self.address, ID_VENDOR, ID_PRODUCT))



class TrafficLightError(Exception):
    """Exception raiesed for errors in the traffic light

    Attributes:
        message -- error message
    """

    def __init__(self, message):
        self.message = message


class MultipleTrafficLightsError(Exception):
    """Exception raised if multiple traffic lights are connected but no address
    is specified

    Attributes:
        message -- error message
    """

    def __init__(self, message):
        self.message = message



from setuptools import setup, find_packages

with open("README.md", "r", encoding='utf-8') as desc_file:
    long_description = desc_file.read()

setup(
    name='cleware-traffic-light',
    version='1.0.2',
    description='Python3 modul and cli tool to controll the Cleware traffic light',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Josh Rost',
    author_email='joshua.s.rost@gmail.com',
    url='https://github.com/joshrost/cleware-traffic-light',
    packages=find_packages(),
    install_requires=['pyusb'],
    entry_points={
        'console_scripts': ['ctl=traffic_light.__main__:main'],
    }
)
